@@ -5,7 +5,7 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.3.13
+# Version   : 0.3.20
 
 from operator import contains
 import  numpy                   as      np
@@ -17,6 +17,11 @@ import  re
 import  os
 from    sklearn.linear_model    import  LinearRegression
 
+import  pandas              as      pd
+import  seaborn             as      sns
+import  matplotlib.pyplot   as      plt
+from    fnmatch             import  fnmatch
+
 class AFM:
 
     '''
@@ -1096,22 +1101,51 @@ class AFM:
             indenter_term = 0
 
         for curve_number in range(0, self.z_approach.shape[1]):
-            _approach = self.approach[:,curve_number]
-            _z = self.z_approach[:,curve_number]
 
-            force = _approach[np.isfinite(_approach)][0] * 1e-9
-            depth = _z[np.isfinite(_approach)][0] * 1e-9
-            ratio = depth / indenter_radius
+            force = self.approach[:,curve_number] * 1e-9
+            depth = self.z_approach[:,curve_number] * 1e-9
 
-            term_1 = (4/3)
-            term_2 = 1 / (1 - poisson_ratio ** 2)
-            term_3 = 1 / force
-            term_4 = np.sqrt(indenter_radius)
-            term_5 = np.absolute(depth) ** (3/2)
+            indent_fit = LinearRegression(fit_intercept = False)
 
-            terms = (term_1* term_3 * term_4 * term_5 - indenter_term) * term_2 
+            x = depth
+            y = force
+
+            x = x[~np.isnan(y)]
+            y = y[~np.isnan(y)]
+
+            y = y[~np.isnan(x)]
+            x = x[~np.isnan(x)]
+
+            x = x[y>0]
+            y = y[y>0]
+
+            y = y[x<0]
+            x = x[x<0]
+            
+            x = x.reshape(-1, 1)
+            y = y.reshape(-1, 1)
+
+            max_depth = np.min(x)
+
+            if len(x) == 0:
+
+                self.modulus[curve_number] = np.nan
+                
+            else:
+
+                indent_fit.fit(x, y)
+                max_force = indent_fit.predict(max_depth.reshape(-1, 1))
+
+                term_1 = (4/3)
+                term_2 = 1 / (1 - poisson_ratio ** 2)
+                term_3 = 1 / max_force
+                term_4 = np.sqrt(indenter_radius)
+                term_5 = np.absolute(max_depth) ** (3/2)
+
+                terms = (term_1* term_3 * term_4 * term_5 - indenter_term) * term_2
+
+                self.modulus[curve_number] = (1 / terms) * 1e-6
 
-            self.modulus[curve_number] = (1 / terms) * 1e-6
         
         if plot_hist is True:
             fig = go.Figure(data=[go.Histogram(x = self.modulus)])
@@ -1171,13 +1205,18 @@ class AFM:
             indent_fit = LinearRegression(fit_intercept = intercept)
 
             x = depth
-            y = np.sqrt(force)
+            y = force
 
             x = x[~np.isnan(y)]
             y = y[~np.isnan(y)]
 
             y = y[~np.isnan(x)]
             x = x[~np.isnan(x)]
+
+            x = x[y>0]
+            y = y[y>0]
+
+            y = np.sqrt(y)
             
             x = x.reshape(-1, 1)
             y = y.reshape(-1, 1)
@@ -1595,7 +1634,7 @@ class AFM:
                 fig.update_layout(scene = dict(
                                 xaxis_title='X Position (um)',
                                 yaxis_title='Y Position (um)',
-                                zaxis_title='Z Position (um)'),
+                                zaxis_title='Z Position (nm)'),
                                 margin=dict(r=20, b=10, l=10, t=10))
 
             fig.show()
